package itso.bank.servlet;

import itso.bank.facade.Bank;
import itso.bank.model.ejb.Account;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.Servlet;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * Servlet to retrieve the details for a given account.
 *
 * Parameters: 
 * <dl>
 * <dt>accountId</dt>
 * <dd>The account number for the account to be shown.</dd>
 * </dl>
 * 
 * Output (request parameters): 
 * <dl>
 * <dt>account</dt>
 * <dd>An instance of the Account class, representing the account
 *     to be displayed.</dd>
 * </dl>
 * 
 * Forwards to:
 * <dl>
 * <dt>accountDetails.jsp</dt>
 * <dd>If the account is found and no other error occurred.</dd>
 * 
 * <dt>showException.jsp</dt>
 * <dd>If the customer is not found, or some other error has occurred.</dd>
 * </dl>
 * 
 * @see itso.bank.model.Account
 */
public class AccountDetails
		extends HttpServlet
		implements Servlet
{
	/**
	 * HTTP GET service method. Calls performTask to service requests.
	 * 
	 * @see performTask(HttpServletRequest req, HttpServletResponse resp)
	 * @see javax.servlet.http.HttpServlet#doGet(HttpServletRequest req,
	 *      HttpServletResponse resp)
	 */
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException
	{
		performTask(req, resp);
	}

	/**
	 * HTTP POST service method. Calls performTask to service requests.
	 * 
	 * @see performTask(HttpServletRequest req, HttpServletResponse resp)
	 * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest req,
	 *      HttpServletResponse resp)
	 */
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException
	{
		performTask(req, resp);
	}

	/**
	 * This method does the actual servlet processing.
	 * 
	 * @see #doGet(HttpServletRequest, HttpServletResponse)
	 * @see #doPost(HttpServletRequest, HttpServletResponse)
	 */
	private void performTask(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
	{
		try
		{
			// parameters
			String accountNumber = req.getParameter("accountId");
			
			// Control logic - Create the new banking facade
			Bank bank = Bank.getBank();
			
			// Retrieve customer and related accounts
			Account account = bank.getAccount(accountNumber);
			
			// Response - Set the request attributes for future rendering
			req.setAttribute("account", account);
			
			// Call the presentation renderer
			ServletContext ctx = getServletContext();
			RequestDispatcher disp = ctx.getRequestDispatcher("accountDetails.jsp");
			disp.forward(req, resp);
		}
		catch (Exception e)
		{
			// set up error information and forward to the error page
			req.setAttribute("message", e.getMessage());
			req.setAttribute("forward", "index.html");
			ServletContext ctx = getServletContext();
			RequestDispatcher disp = ctx.getRequestDispatcher("showException.jsp");
			disp.forward(req, resp);
		}
	}
}
