package itso.bank.servlet;

import itso.bank.facade.Bank;
import itso.bank.model.ejb.Account;
import itso.bank.model.ejb.Customer;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.Servlet;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;


/**
 * Servlet to retrieve the details for a given customer, along with a list
 * of accounts that the customer owns.
 * This servlet also performs a "log on", i.e. the given customer number
 * will be placed on the session, if it matches a customer in the bank.
 *
 * Parameters: 
 * <dl>
 * <dt>customerNumber</dt>
 * <dd>The customer number (SSN) of the user loggin on. If not specified as 
 *     a request parameter, it will be retrieved from the session.</dd>
 * </dl>
 * 
 * Output (request parameters): 
 * <dl>
 * <dt>customer</dt>
 * <dd>An instance of the Customer class, representing customer that is 
 *     currently logged on</dd>
 * 
 * <dt>accounts</dt>
 * <dd>An array of instance of the Account class, representing the accounts
 *     that the current customer owns.</dd>
 * </dl>
 * 
 * Forwards to:
 * <dl>
 * <dt>listAccounts.jsp</dt>
 * <dd>If the customer is found and no other error occurred.</dd>
 * 
 * <dt>showException.jsp</dt>
 * <dd>If the customer is not found, or some other error has occurred.</dd>
 * </dl>
 * 
 * @see itso.bank.model.Customer
 * @see itso.bank.model.Account
 */
public class ListAccounts
		extends HttpServlet
		implements Servlet
{
	/**
	 * HTTP GET service method. Calls performTask to service requests.
	 * 
	 * @see performTask(HttpServletRequest req, HttpServletResponse resp)
	 * @see javax.servlet.http.HttpServlet#doGet(HttpServletRequest req,
	 *      HttpServletResponse resp)
	 */
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException
	{
		performTask(req, resp);
	}

	/**
	 * HTTP POST service method. Calls performTask to service requests.
	 * 
	 * @see performTask(HttpServletRequest req, HttpServletResponse resp)
	 * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest req,
	 *      HttpServletResponse resp)
	 */
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException
	{
		performTask(req, resp);
	}

	/**
	 * This method does the actual servlet processing.
	 * 
	 * @see #doGet(HttpServletRequest, HttpServletResponse)
	 * @see #doPost(HttpServletRequest, HttpServletResponse)
	 */
	private void performTask(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
	{
		try
		{
			// Get input parameter and keep it on the HTTP session
			String customerNumber = req.getParameter("customerNumber");
			HttpSession session = req.getSession();
			
			if (customerNumber == null)
				customerNumber = (String) session.getAttribute("customerNumber");
			else
				session.setAttribute("customerNumber", customerNumber);
			
			// Control logic - Create the new banking facade
			Bank bank = Bank.getBank();
			
			// Retrieve customer and related accounts
			Customer customer = bank.getCustomer(customerNumber);
			Account[] accounts = bank.getAccounts(customerNumber);
			
			// Response - Set the request attributes for future rendering
			req.setAttribute("customer", customer);
			req.setAttribute("accounts", accounts);
			
			// Call the presentation renderer
			ServletContext ctx = getServletContext();
			RequestDispatcher disp = ctx.getRequestDispatcher("listAccounts.jsp");
			disp.forward(req, resp);
		}
		catch (Exception e)
		{
			// set up error information and forward to the error page
			req.setAttribute("message", e.getMessage());
			req.setAttribute("forward", "index.html");
			ServletContext ctx = getServletContext();
			RequestDispatcher disp = ctx.getRequestDispatcher("showException.jsp");
			disp.forward(req, resp);
		}
	}
}
